﻿using System;
using System.IO;
using System.Windows.Forms;

using NAudio.Wave;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 타겟 디렉토리 경로
        /// </summary>
        private string targetDirectoryPath;

        /// <summary>
        /// 타겟 파일 경로
        /// </summary>
        private string targerFilePath;

        /// <summary>
        /// WASAPI 루프백 캡처
        /// </summary>
        private WasapiLoopbackCapture wasapiLoopbackCapture;

        /// <summary>
        /// 웨이브 파일 작성자
        /// </summary>
        private WaveFileWriter waveFileWriter = null;

        /// <summary>
        /// 종료 여부
        /// </summary>
        private bool closing = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.targetDirectoryPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), "NAudio");

            if(!Directory.Exists(this.targetDirectoryPath))
            {
                Directory.CreateDirectory(this.targetDirectoryPath);
            }

            this.targerFilePath = Path.Combine(this.targetDirectoryPath, "result.wav");

            this.wasapiLoopbackCapture = new WasapiLoopbackCapture();

            FormClosing                                 += Form_FormClosing;
            this.recordButton.Click                     += recordButton_Click;
            this.stopButton.Click                       += stopButton_Click;
            this.wasapiLoopbackCapture.DataAvailable    += wasapiLoopbackCapture_DataAvailable;
            this.wasapiLoopbackCapture.RecordingStopped += wasapiLoopbackCapture_RecordingStopped;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼을 닫을 경우 처리하기 - Form_FormClosing(sender, e)

        /// <summary>
        /// 폼을 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.closing = true;
            
            this.wasapiLoopbackCapture.StopRecording();
        }

        #endregion
        #region 레코딩 버튼 클릭시 처리하기 - recordButton_Click(sender, e)

        /// <summary>
        /// 레코딩 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void recordButton_Click(object sender, EventArgs e)
        {
            this.waveFileWriter = new WaveFileWriter(this.targerFilePath, this.wasapiLoopbackCapture.WaveFormat);
            
            this.wasapiLoopbackCapture.StartRecording();

            this.recordButton.Enabled = false; 
            this.stopButton.Enabled   = true;
        }

        #endregion
        #region 중단 버튼 클릭시 처리하기 - stopButton_Click(sender, e)

        /// <summary>
        /// 중단 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.wasapiLoopbackCapture.StopRecording();
        }

        #endregion
        #region WASAPI 루프백 캡처 데이터 이용 가능시 처리하기 - wasapiLoopbackCapture_DataAvailable(sender, e)

        /// <summary>
        /// WASAPI 루프백 캡처 데이터 이용 가능시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void wasapiLoopbackCapture_DataAvailable(object sender, WaveInEventArgs e)
        {
            this.waveFileWriter.Write(e.Buffer, 0, e.BytesRecorded);

            if(this.waveFileWriter.Position > this.wasapiLoopbackCapture.WaveFormat.AverageBytesPerSecond * 30)
            {
                this.wasapiLoopbackCapture.StopRecording();
            }
        }

        #endregion
        #region WASAPI 루프백 캡처 레코딩 중단시 처리하기 - wasapiLoopbackCapture_RecordingStopped(sender, e)

        /// <summary>
        /// WASAPI 루프백 캡처 레코딩 중단시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void wasapiLoopbackCapture_RecordingStopped(object sender, StoppedEventArgs e)
        {
            this.waveFileWriter?.Dispose();

            this.waveFileWriter = null;

            this.recordButton.Enabled = true;
            this.stopButton.Enabled   = false;

            if(this.closing) 
            { 
                this.wasapiLoopbackCapture.Dispose();
            }
        }

        #endregion
    }
}